//Code to control a counter on a serial port. The daemon counts connection
//attempts to the socket /tmp/ctrsock. See the included php-file for an example
//how to use it.

//This code is licensed under the GNU GPL, V2 or later. See LICENSE for
//more information. Copyright 2007 J. Domburg <projectmechctr@jeroen.ietsmet.nl>

#define SOCKPATH "/tmp/ctrsock"


#include <stdio.h>
#include <stdlib.h>
#include <asm/termios.h>
#include <asm/ioctls.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <unistd.h>
#include <netinet/in.h>

//Turns off the power to the counter       
int counter_setopen(int f) {
    int t;
    t=TIOCM_RTS; ioctl(f,TIOCMBIC,&t);
    t=TIOCM_DTR; ioctl(f,TIOCMBIS,&t);
}

//Turns on the power to the counter
int counter_setclose(int f) {
    int t;
    t=TIOCM_DTR|TIOCM_RTS; ioctl(f,TIOCMBIS,&t);
}

//Main
int main(int argc, char** argv) {
    int f;
    int pid;
    int s,as,t;
    long topen;
    long tclose;
    FILE *pidfile;
    struct sockaddr servaddr;
    topen=20000; tclose=30000;
    
    if (argc<2) {
	printf("Usage: %s port [opentime [closetime]]\nwith opentime and closetime in uSec\n",argv[0]);
	exit(0);
    }
    
    
    //parse arguments
    if (argc>2) topen=atol(argv[2]);
    if (argc>3) tclose=atol(argv[2]);
    f=open(argv[1],O_RDWR|O_NOCTTY);
    if (f<1) { perror("Opening serial port"); exit(0); }

    //create unix domain socket
    servaddr.sa_family=AF_UNIX;
    strcpy(servaddr.sa_data,SOCKPATH);
    unlink(servaddr.sa_data);
    s=socket(AF_UNIX, SOCK_STREAM, 0);
    if (s<1) { perror("Creating socket"); exit(0); }
    t=bind(s,&servaddr,sizeof(servaddr));
    if (t!=0) { perror("Binding socket"); exit(0); }
    t=chmod(SOCKPATH,0666);
    if (t!=0) { perror("Chmodding socket"); exit(0); }
    t=listen(s,200);
    if (t!=0) { perror("Listen() on socket"); exit(0); }
    
    //fork
    if ((pid=fork())!=0) {
	//create pidfile
	pidfile=fopen("/var/run/ctrd.pid","w");
	if (pidfile==NULL) { perror("opening pidfile"); exit(0); }
	fprintf(pidfile,"%i\n",pid);
	fclose(pidfile);
	exit(0);
    }

    
    //main loop
    while (as=accept(s,NULL,0)) {
	close(as);
	counter_setclose(f);
	usleep(tclose);
	counter_setopen(f);
	usleep(topen);
    }
}

